/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.nachrichten.postfach.bayernid.proxy;

import org.apache.hc.client5.http.impl.classic.HttpClientBuilder;
import org.apache.hc.client5.http.impl.io.BasicHttpClientConnectionManager;
import org.apache.hc.client5.http.io.HttpClientConnectionManager;
import org.apache.hc.client5.http.socket.ConnectionSocketFactory;
import org.apache.hc.client5.http.socket.PlainConnectionSocketFactory;
import org.apache.hc.client5.http.ssl.SSLConnectionSocketFactory;
import org.apache.hc.core5.http.config.RegistryBuilder;
import org.springframework.boot.ssl.SslBundles;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.oxm.jaxb.Jaxb2Marshaller;
import org.springframework.ws.transport.http.HttpComponents5MessageSender;

import akdb.bsp.postkorb.komm.webservice.ObjectFactory;
import de.ozgcloud.nachrichten.postfach.bayernid.proxy.akdb.BayernIdRemoteService;
import de.ozgcloud.nachrichten.postfach.bayernid.proxy.message.BayernIdMessageMapper;
import lombok.RequiredArgsConstructor;

@Configuration
@RequiredArgsConstructor
class BayernIdProxyConfiguration {

	private final SslBundles sslBundles;
	private final BayernIdProperties properties;

	@Bean
	Jaxb2Marshaller createJaxb2Marshaller() {
		var marshaller = new Jaxb2Marshaller();
		marshaller.setContextPaths("akdb.bsp.postkorb.komm.webservice", "de.akdb.egov.bsp.nachrichten");
		return marshaller;
	}

	@Bean
	BayernIdRemoteService soapHttpClient(BayernIdMessageMapper messageMapper, Jaxb2Marshaller marshaller) {
		var bayernIdRemoteService = new BayernIdRemoteService(messageMapper, new ObjectFactory());
		bayernIdRemoteService.setMarshaller(marshaller);
		bayernIdRemoteService.setUnmarshaller(marshaller);
		bayernIdRemoteService.setDefaultUri(properties.getServer());
		bayernIdRemoteService.setMessageSender(createMessageSender());
		return bayernIdRemoteService;
	}

	HttpComponents5MessageSender createMessageSender() {
		var httpClient = HttpClientBuilder.create()
				.setConnectionManager(createConnectionManager())
				.addRequestInterceptorFirst(new HttpComponents5MessageSender.RemoveSoapHeadersInterceptor())
				.build();
		return new HttpComponents5MessageSender(httpClient);
	}

	private HttpClientConnectionManager createConnectionManager() {
		var socketFactoryRegistry = RegistryBuilder.<ConnectionSocketFactory>create().register("http", new PlainConnectionSocketFactory());
		if (properties.isUseSsl()) {
			socketFactoryRegistry.register("https", createSslConnectionSocketFactory());
		}
		return new BasicHttpClientConnectionManager(socketFactoryRegistry.build());
	}

	private SSLConnectionSocketFactory createSslConnectionSocketFactory() {
		return new SSLConnectionSocketFactory(sslBundles.getBundle("bayern-id-ca").createSslContext());
	}

}
